@@ -1,15 +1,16 @@
+"""VCF and SnpEff/SnpSift parsing functions"""
 import logging
 import os
 import re
 from enum import Enum
 from operator import itemgetter
 from pathlib import Path
-from typing import Dict, Tuple, List, Optional, Iterable
+from typing import Dict, Tuple, List, Optional, Iterable, Union
 
 import pandas as pd
-import numpy as np
 from pydantic import BaseModel
 
+from xlavir.qc import QualityRequirements
 from xlavir.util import try_parse_number, find_file_for_each_sample
 
 logger = logging.getLogger(__name__)
@@ -131,6 +132,16 @@ variant_summary_cols = [
         'Mean AF',
         'Mean/average alternate allele frequency of mutation in all samples.'
     ),
+    (
+        'nt_pos',
+        'Nucleotide Position',
+        'Nucleotide position of mutation with respect to reference genome.'
+    ),
+    (
+        'aa_pos',
+        'Amino Acid Position',
+        'Amino acid position of mutation in reference genome gene.'
+    )
 ]
 
 BCFTOOLS_STATS_GLOB_PATTERNS = [
@@ -155,26 +166,40 @@ class VariantStats(BaseModel):
 class VariantCaller(Enum):
     iVar = 'iVar'
     Longshot = 'Longshot'
+    Nanopolish = 'nanopolish'
+    Medaka = 'medaka'
 
 
 VCF_GLOB_PATTERNS = [
+    '**/nanopolish/*.pass.vcf.gz',
     '**/ivar/*.vcf.gz',
+    '**/*.filt.no_fs.vcf',
+    '**/*.longshot.vcf',
     '**/*.vcf',
 ]
 
 VCF_SAMPLE_NAME_CLEANUP = [
-    re.compile(r'\.vcf(\.gz)?$'),
+    re.compile(r'(\.pass)?\.vcf(\.gz)?$'),
     re.compile(r'\.AF0\.\d+(\.filt)?'),
     re.compile(r'\.0\.\d+AF(\.filt)?'),
+    re.compile(r'\.medaka'),
+    re.compile(r'\.longshot'),
+    re.compile(r'\.snpeff'),
+    re.compile(r'\.no_fs'),
 ]
 
 SNPSIFT_GLOB_PATTERNS = [
     '**/ivar/**/*.snpSift.table.txt',
+    '**/ivar/**/*.snpsift.table.txt',
+    '**/ivar/**/*.snpsift.txt',
     '**/*.snpSift.table.txt',
+    '**/*.snpsift.table.txt',
+    '**/*.snpsift.txt',
 ]
 
 SNPSIFT_SAMPLE_NAME_CLEANUP = [
-    re.compile(r'\.snpSift\.table\.txt$'),
+    re.compile(r'\.snp[sS]ift\.table\.txt$'),
+    re.compile(r'\.snp[sS]ift\.txt$'),
     re.compile(r'\.AF0\.\d+(\.filt)?'),
     re.compile(r'\.0\.\d+AF(\.filt)?'),
 ]
@@ -213,6 +238,10 @@ def read_vcf(vcf_file: Path) -> Tuple[str, pd.DataFrame]:
         for line in fh:
             if line.startswith('##source='):
                 variant_caller = line.strip().replace('##source=', '')
+            if line.startswith('##nanopolish'):
+                variant_caller = 'nanopolish'
+            if line.startswith('##medaka_version'):
+                variant_caller = 'medaka'
             if line.startswith('#CHROM'):
                 vcf_cols = line[1:].strip().split('\t')
                 break
@@ -220,6 +249,7 @@ def read_vcf(vcf_file: Path) -> Tuple[str, pd.DataFrame]:
                            comment='#',
                            header=None,
                            names=vcf_cols)
+        df = df[~df.duplicated(['CHROM', 'POS', 'ID', 'REF', 'ALT', 'FILTER'], keep='first')]
     return variant_caller, df
 
 
@@ -230,24 +260,33 @@ def parse_aa(gene: str,
              aa_pos: int,
              snpeff_aa: str,
              effect: str) -> str:
-    m = re.match(r'p\.([a-zA-Z]+)(\d+)([a-zA-Z]+)', snpeff_aa)
-    if snpeff_aa == '.' or m is None:
+    if snpeff_aa == '.':
         return f'{ref}{nt_pos}{alt}'
+    m = re.match(r'p\.([a-zA-Z]+)(\d+)([a-zA-Z]+)', snpeff_aa)
+    if m is None and snpeff_aa.startswith('p.'):
+        aa_str = snpeff_aa[2:].upper()
+        for aa3, aa1 in aa_codes.items():
+            aa_str = aa_str.replace(aa3, aa1)
+        if aa_str.endswith('DEL'):
+            aa_str = aa_str.replace('DEL', 'del')
+        return f'{gene}:{aa_str} ({ref}{nt_pos}{alt})'
     ref_aa, aa_pos_str, alt_aa = m.groups()
     ref_aa = get_aa(ref_aa)
 
     if effect == 'stop_lost':
         alt_aa = get_aa(alt_aa.replace('ext', ''))
-        return f'{ref}{nt_pos}{alt}({gene}:{ref_aa}{aa_pos_str}{alt_aa}[stop_lost])'
+        return f'{gene}:{ref_aa}{aa_pos_str}{alt_aa} ({ref}{nt_pos}{alt} [stop_lost])'
     if effect == 'frameshift_variant':
-        return f'{ref}{nt_pos}{alt}({gene}:{ref_aa}{aa_pos_str}[FRAMESHIFT])'
+        return f'{gene}:{ref_aa}{aa_pos_str} ({ref}{nt_pos}{alt} [FRAMESHIFT])'
     if effect == 'conservative_inframe_deletion':
-        return f'{ref}{nt_pos}{alt}({gene}:{ref_aa}{aa_pos_str}{alt_aa})'
+        return f'{gene}:{ref_aa}{aa_pos_str}{alt_aa} ({ref}{nt_pos}{alt})'
     if effect == 'disruptive_inframe_deletion':
-        return f'{ref}{nt_pos}{alt}({gene}:{ref_aa}{aa_pos_str}{alt_aa}[disruptive_inframe_deletion])'
+        return f'{gene}:{ref_aa}{aa_pos_str}{alt_aa} ({ref}{nt_pos}{alt} [disruptive_inframe_deletion])'
 
     alt_aa = get_aa(alt_aa)
-    return f'{ref}{nt_pos}{alt}({gene}:{ref_aa}{aa_pos_str}{alt_aa})'
+    if alt_aa == ref_aa:
+        return f'{ref}{nt_pos}{alt}'
+    return f'{gene}:{ref_aa}{aa_pos_str}{alt_aa} ({ref}{nt_pos}{alt})'
 
 
 def get_aa(s: str) -> str:
@@ -265,6 +304,7 @@ def get_aa(s: str) -> str:
 def simplify_snpsift(df: pd.DataFrame, sample_name: str) -> Optional[pd.DataFrame]:
     if df.empty:
         return None
+    df = df[~df.duplicated(keep='first')]
     field_names = set()
     series = []
     for c in df.columns:
@@ -278,6 +318,16 @@ def simplify_snpsift(df: pd.DataFrame, sample_name: str) -> Optional[pd.DataFram
             AF.name = 'AF'
             series += [REF_AC, ALT_AC, AF]
             continue
+        if c == 'SR':
+            df_ac = df[c].str.split(',', expand=True)
+            REF_AC = df_ac[0].astype(int) + df_ac[1].astype(int)
+            REF_AC.name = 'REF_AC'
+            ALT_AC = df_ac[2].astype(int) + df_ac[3].astype(int)
+            ALT_AC.name = 'ALT_AC'
+            AF = ALT_AC / (REF_AC + ALT_AC)
+            AF.name = 'AF'
+            series += [REF_AC, ALT_AC, AF]
+            continue
         idx = c.find('[*].')
         if idx > 0:
             new_series_name = c[idx + 4:].lower()
@@ -319,15 +369,34 @@ def parse_ivar_vcf(df: pd.DataFrame, sample_name: str = None) -> Optional[pd.Dat
             raise ValueError(f'Sample name is not defined for VCF: shape={df.shape}; columns={df.columns}')
     pos_fmt_val = {}
     for row in df.itertuples():
+        # An iVar VCF has the DP, total depth, in the INFO field, you cannot sum the alt and ref depths since the ref
+        # depth only applies to the first base in the ref allele rather than over the entire allele so the depth can be
+        # misleading for deletions
+        # See iVar issue: https://github.com/andersen-lab/ivar/issues/86
+        infos = parse_vcf_info(row.INFO)
+        total_dp = infos['DP']
         ks = row.FORMAT.split(':')
         vs = row[-1].split(':')
-        pos_fmt_val[row.POS] = {k: try_parse_number(v) for k, v in zip(ks, vs)}
+        record: Dict[str, Union[float, int, str]] = {k: try_parse_number(v) for k, v in zip(ks, vs)}
+        ref_dp = record['REF_DP']
+        alt_dp = record['ALT_DP']
+        # if the sum of the ref and alt dp does not equal the total dp reported by iVar then recalculate the ref dp
+        # since it is likely only reporting the ref dp for the first base of a longer deletion. SNPs should be fine.
+        sum_ref_alt_dp = ref_dp + alt_dp
+        if sum_ref_alt_dp != total_dp:
+            record['REF_DP'] = total_dp - alt_dp
+            logger.warning(
+                f'iVar VCF for sample "{sample_name}" contains a variant at position {row.POS} where sum of ref allele '
+                f'depth ({ref_dp}) and alt allele depth ({alt_dp}) does not equal the total depth ({total_dp}) '
+                f'reported by iVar (i.e. {ref_dp} + {alt_dp} = {sum_ref_alt_dp}; {sum_ref_alt_dp} != {total_dp}). Ref '
+                f'allele is "{row.REF}", alt allele is "{row.ALT}". iVar alt allele frequency: {record["ALT_FREQ"]}')
+        pos_fmt_val[row.POS] = record
+        pos_fmt_val[row.POS]['DP'] = total_dp
     df_ivar_info = pd.DataFrame(pos_fmt_val).transpose()
     df_ivar_info.index.name = 'POS'
     df_ivar_info.reset_index(inplace=True)
     df_merge = pd.merge(df, df_ivar_info, on='POS')
     df_merge['sample'] = sample_name
-    df_merge['DP'] = df_merge.ALT_DP + df_merge.REF_DP
     return df_merge.drop(columns=['ID', 'INFO', 'QUAL', 'FILTER', 'FORMAT', df.columns[-1], 'GT'])
 
 
@@ -372,6 +441,69 @@ def parse_longshot_vcf(df: pd.DataFrame, sample_name: str = None) -> Optional[pd
     return df_merge.loc[:, cols_to_keep]
 
 
+def parse_medaka_vcf(df: pd.DataFrame, sample_name: str = None, qc_reqs: QualityRequirements = None) -> Optional[pd.DataFrame]:
+    if df.empty:
+        return None
+    if not sample_name:
+        sample_name = df.columns[-1] if df.columns[-1] != 'SAMPLE' else None
+        if sample_name is None:
+            raise ValueError(f'Sample name is not defined for VCF: shape={df.shape}; columns={df.columns}')
+    pos_info_val = {}
+    for row in df.itertuples():
+        # if there is no variant INFO available then this isn't the right file
+        if row.INFO == '.':
+            return None
+        infos = parse_vcf_info(row.INFO)
+        # no DP INFO? skip this file
+        if 'DP' not in infos:
+            return None
+        if infos['DP'] < qc_reqs.low_coverage_threshold:
+            continue
+        ac_ref_fwd, ac_ref_rev, ac_alt_fwd, ac_alt_rev = infos['SR']
+        infos['REF_DP'] = ac_ref_fwd + ac_ref_rev
+        infos['ALT_DP'] = ac_alt_fwd + ac_alt_rev
+        pos_info_val[row.POS] = infos
+    df_medaka_info = pd.DataFrame(pos_info_val).transpose()
+    df_medaka_info.index.name = 'POS'
+    df_medaka_info.reset_index(inplace=True)
+    df_merge = pd.merge(df, df_medaka_info, on='POS')
+    df_merge['sample'] = sample_name
+    df_merge = df_merge[df_merge.DP > 0]
+    df_merge['ALT_FREQ'] = df_merge.ALT_DP / (df_merge.ALT_DP + df_merge.REF_DP)
+    cols_to_keep = list({col for col, _, _ in variants_cols} & set(df_merge.columns))
+    return df_merge.loc[:, cols_to_keep]
+
+
+def parse_nanopolish_vcf(df: pd.DataFrame, sample_name: str = None) -> Optional[pd.DataFrame]:
+    if df.empty:
+        return None
+    if not sample_name:
+        sample_name = df.columns[-1] if df.columns[-1] != 'sample' else None
+        if sample_name is None:
+            raise ValueError(f'Sample name is not defined for VCF: shape={df.shape}; columns={df.columns}')
+    pos_info_val = {}
+    for row in df.itertuples():
+        infos = parse_vcf_info(row.INFO)
+        fwd_ref, rev_ref, fwd_alt, rev_alt = infos['StrandSupport']
+        allele_count = infos['AlleleCount']
+        if allele_count > 1:
+            raise NotImplementedError(f'Handling of allele count of {allele_count} is not supported. '
+                                      f'Only allele counts of 1 are supported.')
+        infos['DP'] = fwd_ref + fwd_alt + rev_ref + rev_alt
+        infos['REF_DP'] = fwd_ref + rev_ref
+        infos['ALT_DP'] = fwd_alt + rev_alt
+        pos_info_val[row.POS] = infos
+    df_nanopolish_info = pd.DataFrame(pos_info_val).transpose()
+    df_nanopolish_info.index.name = 'POS'
+    df_nanopolish_info.reset_index(inplace=True)
+    df_merge = pd.merge(df, df_nanopolish_info, on='POS')
+    df_merge['sample'] = sample_name
+    df_merge = df_merge[df_merge.DP > 0]
+    df_merge['ALT_FREQ'] = df_merge.ALT_DP / df_merge.DP
+    cols_to_keep = list({col for col, _, _ in variants_cols} & set(df_merge.columns))
+    return df_merge.loc[:, cols_to_keep]
+
+
 def parse_vcf_info(s: str) -> dict:
     out = {}
     for x in s.split(';'):
@@ -382,7 +514,7 @@ def parse_vcf_info(s: str) -> dict:
     return out
 
 
-def get_info(basedir: Path) -> Dict[str, pd.DataFrame]:
+def get_info(basedir: Path, qc_reqs: QualityRequirements) -> Dict[str, pd.DataFrame]:
     sample_vcf = find_file_for_each_sample(basedir=basedir,
                                            glob_patterns=VCF_GLOB_PATTERNS,
                                            sample_name_cleanup=VCF_SAMPLE_NAME_CLEANUP,
@@ -396,19 +528,33 @@ def get_info(basedir: Path) -> Dict[str, pd.DataFrame]:
                 sample_dfvcf[sample] = df_parsed_ivar_vcf
             else:
                 logger.warning(f'Sample "{sample}" has no entries in VCF "{vcf_path}"')
+        elif variant_caller.startswith(VariantCaller.Medaka.value):
+            df_medaka_vcf = parse_medaka_vcf(df_vcf, sample, qc_reqs)
+            if df_medaka_vcf is not None:
+                sample_dfvcf[sample] = df_medaka_vcf
+            else:
+                logger.warning(f'Sample "{sample}" has no entries in VCF "{vcf_path}"')
         elif variant_caller.startswith(VariantCaller.Longshot.value):
-            df_parsed_ivar_vcf = parse_longshot_vcf(df_vcf, sample)
-            if df_parsed_ivar_vcf is not None:
-                sample_dfvcf[sample] = df_parsed_ivar_vcf
+            df_longshot_vcf = parse_longshot_vcf(df_vcf, sample)
+            if df_longshot_vcf is not None:
+                sample_dfvcf[sample] = df_longshot_vcf
+            else:
+                logger.warning(f'Sample "{sample}" has no entries in VCF "{vcf_path}"')
+        elif variant_caller.startswith(VariantCaller.Nanopolish.value):
+            df_nanopolish_vcf = parse_nanopolish_vcf(df_vcf, sample)
+            if df_nanopolish_vcf is not None:
+                sample_dfvcf[sample] = df_nanopolish_vcf
             else:
                 logger.warning(f'Sample "{sample}" has no entries in VCF "{vcf_path}"')
         else:
-            raise NotImplementedError()
+            logger.warning(f'Sample "{sample}" VCF file "{vcf_path}" with variant_caller={variant_caller} not supported. Skipping...')
 
     sample_snpsift = find_file_for_each_sample(basedir=basedir,
                                                glob_patterns=SNPSIFT_GLOB_PATTERNS,
                                                sample_name_cleanup=SNPSIFT_SAMPLE_NAME_CLEANUP,
                                                single_entry_selector_func=snpsift_selector)
+    if not sample_snpsift:
+        logger.warning(f'No SnpSift tables found in "{basedir}" using glob patterns "{SNPSIFT_GLOB_PATTERNS}"')
     sample_dfsnpsift = {}
     for sample, snpsift_path in sample_snpsift.items():
         df_snpsift = simplify_snpsift(pd.read_table(snpsift_path), sample)
@@ -442,6 +588,20 @@ def to_dataframe(dfs: Iterable[pd.DataFrame]) -> pd.DataFrame:
     return df.rename(columns={x: y for x, y, _ in variants_cols})
 
 
+def get_nt_position_int(s: str) -> int:
+    """Get the nucleotide position from the mutation string
+
+    >>> get_nt_position_int('A1879G')
+    1879
+    >>> get_nt_position_int('S:Y145_H146del (TATTACC21992T)')
+    21992
+    """
+    if ':' in s:
+        return int(re.sub(r'.*\([AGTC]+(\d+).*', r'\1', s))
+    else:
+        return int(re.sub(r'[AGTC]+(\d+).*', r'\1', s))
+
+
 def to_variant_pivot_table(df: pd.DataFrame) -> pd.DataFrame:
     df_vars = df.copy()
     df_vars.reset_index(inplace=True)
@@ -451,8 +611,9 @@ def to_variant_pivot_table(df: pd.DataFrame) -> pd.DataFrame:
                               values='Alternate Allele Frequency',
                               aggfunc='first',
                               fill_value=0.0)
+    nt_positions = [get_nt_position_int(x) for x in df_pivot.columns]
     pivot_cols = list(zip(df_pivot.columns,
-                          df_pivot.columns.str.replace(r'[A-Z]+(\d+).*', r'\1').astype(int)))
+                          nt_positions))
     pivot_cols.sort(key=itemgetter(1))
     return df_pivot[[x for x, y in pivot_cols]]
 
@@ -460,15 +621,22 @@ def to_variant_pivot_table(df: pd.DataFrame) -> pd.DataFrame:
 def to_summary(df: pd.DataFrame) -> pd.DataFrame:
     df_vars = df.copy()
     df_vars.reset_index(inplace=True)
-
+    logger.debug(f'df_vars columns: {df_vars.columns}')
     df_summary = df_vars.groupby('Mutation', sort=False).agg(
-        n_samples=pd.NamedAgg(column='Sample', aggfunc='size'),
-        samples=pd.NamedAgg(column='Sample', aggfunc=lambda x: '; '.join(x)),
-        min_depth=pd.NamedAgg(column='Alternate Allele Depth', aggfunc='min'),
-        max_depth=pd.NamedAgg(column='Alternate Allele Depth', aggfunc='max'),
-        mean_depth=pd.NamedAgg(column='Alternate Allele Depth', aggfunc=lambda x: sum(x) / len(x)),
-        min_af=pd.NamedAgg(column='Alternate Allele Frequency', aggfunc='min'),
-        max_af=pd.NamedAgg(column='Alternate Allele Frequency', aggfunc='max'),
-        mean_af=pd.NamedAgg(column='Alternate Allele Frequency', aggfunc=lambda x: sum(x) / len(x)),
+        n_samples=('Sample', 'size'),
+        samples=('Sample', lambda x: '; '.join(x)),
+        gene=('Gene', 'first'),
+        effect=('Variant Effect', 'first'),
+        impact=('Variant Impact', 'first'),
+        aa=('Amino Acid Change', 'first'),
+        min_depth=('Alternate Allele Depth', 'min'),
+        max_depth=('Alternate Allele Depth', 'max'),
+        mean_depth=('Alternate Allele Depth', lambda x: sum(x) / len(x)),
+        min_af=('Alternate Allele Frequency', 'min'),
+        max_af=('Alternate Allele Frequency', 'max'),
+        mean_af=('Alternate Allele Frequency', lambda x: sum(x) / len(x)),
+        nt_pos=('Position', 'first'),
+        aa_pos=('Amino Acid Position', 'first')
     )
-    return df_summary.rename(columns={x: y for x, y, _ in variant_summary_cols})
+    df_summary.sort_values('nt_pos', inplace=True)
+    return df_summary.rename(columns={x: y for x, y, _ in (variant_summary_cols + variants_cols)})
